/* locdbase.cc -- a location database for mobile, wireless routing */

/*

  Copyright (C) 2000 President and Fellows of Harvard College

  All rights reserved.

  NOTICE: This software is provided "as is", without any warranty,
  including any implied warranty for merchantability or fitness for a
  particular purpose.  Under no circumstances shall Harvard University
  or its faculty, staff, students or agents be liable for any use of,
  misuse of, or inability to use this software, including incidental
  and consequential damages.

  License is hereby given to use, modify, and redistribute this
  software, in whole or in part, for any commercial or non-commercial
  purpose, provided that the user agrees to the terms of this
  copyright notice, including disclaimer of warranty, and provided
  that this copyright notice, including disclaimer of warranty, is
  preserved in the source code and documentation of anything derived
  from this software.  Any redistributor of this software or anything
  derived from this software assumes responsibility for ensuring that
  any parties to whom such a redistribution is made are fully aware of
  the terms of this license and disclaimer.

  Author: Brad Karp, Harvard University EECS, July, 1999

*/

/* This database is a shim, and uses the simulator's global knowledge of
   topology and node placement to find the true position of the destination. */

#include "locdbase.h"

#include <assert.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>

static class LocDbaseClass : public TclClass {
public:
	LocDbaseClass() : TclClass("LocDbase") {} 
	TclObject *create(int, const char * const *) {
		return (new LocDbase);
	}
} class_locdbase;

LocDbase::LocDbase() :
  table_(NULL), nn_(0)
{
}

LocDbase::~LocDbase()
{
  delete[] table_;
}

int LocDbase::command(int argc, const char * const * argv)
{
  int i;

  if (argc == 3) {
    if (strcmp(argv[1], "nnodes") == 0) {
      assert(table_ == NULL);
      nn_ = atoi(argv[2]);
      table_ = new (MobileNode *)[nn_];
      for (i = 0; i < nn_; i++)
	table_[i] = NULL;
      return(TCL_OK);
    }
  }
  else if (argc == 4) {
    if (strcmp(argv[1], "register") == 0) {
      int addr = atoi(argv[2]);
      MobileNode *mn = (MobileNode *) TclObject::lookup(argv[3]);
      assert((addr > 0) && (addr < nn_));
      table_[addr] = mn;
      return(TCL_OK);
    }
  }
  return TclObject::command(argc, argv);
}

inline MobileNode *LocDbase::nlookup(int addr)
{
  assert((addr > 0) && (addr < nn_));
  return table_[addr];
}
